﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8.8
@EndOfDesignText@
'This Library is the implementation of Google's Cloud TTS: https://cloud.google.com/text-to-speech/docs/reference/rest/

'Events:
#Event: StartedSpeaking
#Event: onComplete
#Event: onError (errorMessage As String)
#Event: AvailableVoices (mapVoiceData as Map)


Sub Class_Globals
	#IF B4A
		Private audio As MediaPlayer
	#Else IF B4J
		Private audio As AudioClip
	#End If

    Private CloudTTS_Parent As Object
	Private CloudTTS_Event_Name As String
	Private API_KEY As String
    Private CloudTTS_IsInitialized As Boolean=False
	Public strPrevToSpeak As String
	
	Private LanguageCode As String="en-US"
	Private LanguageName As String="en-US-Standard-B"
	
	
	Private LanguageGender As String="MALE"
	Public LanguageGenderMale As String="MALE"	'Returns the srting for Male voices
	Public LanguageGenderFemale As String="FEMALE"	'Returns the srting for FeMale voices
	
End Sub


'[Method] - initializes Cloud TTS
'
'CloudTTS_Parent = pointer to parent module (module creating instance of this class)
'CloudTTS_Event_Name = event name stub
'API_KEY = The API key for your app, which can be received from console.cloud.google.com
Public Sub Initialize(Cloud_TTS_Parent As Object, Cloud_TTS_Event_Name As String, Cloud_TTS_API_KEY As String)

    'Quit if already initialized
	If CloudTTS_IsInitialized Then Return
	
	CloudTTS_Parent=Cloud_TTS_Parent
	CloudTTS_Event_Name=Cloud_TTS_Event_Name
	API_KEY=Cloud_TTS_API_KEY
	
	#IF B4A
		audio.Initialize2("audio")
	#Else If B4J
		audio.Initialize
	#End If
	
	'Flag object is initialized
	CloudTTS_IsInitialized = True
    
End Sub

'Returns True if the CloudTTS Object is initialized. Else, returns False
'Public Sub IsInitialized As Boolean
'	Return CloudTTS_IsInitialized
'End Sub





'[Method] - Lists all the available voices
'Calls the method CloudTTS_AvailableVoices when all the voices are retrived
'Returns Null if error
'Documentation: https://cloud.google.com/text-to-speech/docs/reference/rest/v1/voices/list
'Returns a map carrying all the voices in the following format:
'{
'  "voices": [
'    {
'      "languageCodes": [
'        "da-DK"
'      ],
'      "name": "da-DK-Wavenet-A",
'      "ssmlGender": "FEMALE",
'      "naturalSampleRateHertz": 24000
'    },
'    {
'      "languageCodes": [
'        "de-DE"
'      ],
'      "name": "de-DE-Wavenet-A",
'      "ssmlGender": "FEMALE",
'      "naturalSampleRateHertz": 24000
'    }
'	]
'}
Public Sub ListAllAvailableVoices()
	Dim j As HttpJob:j.Initialize("ListAllAvailableVoices", Me) 'name is empty as it is no longer needed
	j.Tag="ListAllAvailableVoices"
	j.Download("https://texttospeech.googleapis.com/v1/voices?key="&API_KEY)
'	Wait For (j) JobDone(j As HttpJob)
'	If j.Success Then
'		Dim jsonParser As JSONParser
'		jsonParser.Initialize(j.GetString)
'		Dim mapVoiceData As Map=jsonParser.NextObject
'		CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_AvailableVoices", mapVoiceData)
'		j.Release
'		Return (mapVoiceData)
'	Else
'		CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_AvailableVoices", Null)
'		j.Release
'		Return Null
'	End If
End Sub

'[Method] - Lists all the available voices in a particular language
'Calls the method CloudTTS_AvailableVoices when all the voices are retrived
'Returns Null if error
'Documentation: https://cloud.google.com/text-to-speech/docs/reference/rest/v1/voices/list
'Returns a map carrying all the voices in the following format:
'{
'  "voices": [
'    {
'      "languageCodes": [
'        "da-DK"
'      ],
'      "name": "da-DK-Wavenet-A",
'      "ssmlGender": "FEMALE",
'      "naturalSampleRateHertz": 24000
'    },
'    {
'      "languageCodes": [
'        "de-DE"
'      ],
'      "name": "de-DE-Wavenet-A",
'      "ssmlGender": "FEMALE",
'      "naturalSampleRateHertz": 24000
'    }
'	]
'}
Public Sub ListAllAvailableVoices2(Cloud_TTS_LangCode As String)
	Dim j As HttpJob:j.Initialize("ListAllAvailableVoices", Me) 'name is empty as it is no longer needed
	j.Tag="ListAllAvailableVoices"
	j.Download("https://texttospeech.googleapis.com/v1/voices?key="&API_KEY&"&languageCode="&Cloud_TTS_LangCode)
'	Wait For (j) JobDone(j As HttpJob)
'	If j.Success Then
'		Try
'			Dim jsonParser As JSONParser
'			jsonParser.Initialize(j.GetString)
'			Dim mapVoiceData As Map=jsonParser.NextObject
'			CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_AvailableVoices", mapVoiceData)
'			j.Release
'			Return (mapVoiceData)
'		Catch
'			CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_AvailableVoices", Null)
'			j.Release
'			Return Null
'		End Try
'	Else
'		CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_AvailableVoices", Null)
'		j.Release
'		Return Null
'	End If
End Sub


'Sets the Voice for TTS to speak
'lanCode = The Language code, dafault is "en-US"
Public Sub SetVoice(lanCode As String)
	LanguageCode=lanCode
End Sub
'Sets the Voice for TTS to speak
'lanCode = The Language code, dafault is "en-US"
'VoiceGender = The Voice Gender: MALE, FEMALE
Public Sub SetVoice2(lanCode As String, VoiceGender As String)
	LanguageCode=lanCode
	LanguageGender=VoiceGender
End Sub
'Sets the Voice for TTS to speak
'lanName = The Language name
'VoiceGender = The Voice Gender: MALE, FEMALE
Public Sub SetVoice3(lanName As String, VoiceGender As String)
	LanguageName=lanName
	LanguageGender=VoiceGender
	SetVoice(LanguageName.SubString2(0, LanguageName.IndexOf2("-",LanguageName.IndexOf("-")+1)))
End Sub


'Checks if TTS is currently Speaking
Public Sub IsSpeaking As Boolean
	Return audio.IsPlaying
End Sub







Sub JobDone (Job As HttpJob)
	If Job.Tag="ListAllAvailableVoices" Then
		'For listing the available voices
		If Job.Success Then
			Try
				Dim jsonParser As JSONParser
				jsonParser.Initialize(Job.GetString)
				Dim mapVoiceData As Map=jsonParser.NextObject
				CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_AvailableVoices", mapVoiceData)
			Catch
				CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_AvailableVoices", Null)
			End Try
		Else
			CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_AvailableVoices", Null)
		End If
	Else If Job.Tag="speak" Then
		'For speaking the text
		If Job.Success Then
			Try
				Dim jsonParser As JSONParser
				jsonParser.Initialize(Job.GetString)
				Dim strAudio As String=jsonParser.NextObject.Get("audioContent")
				
			    Dim su As StringUtils ' jStringUtils
			    Dim decoded() As Byte = su.DecodeBase64(strAudio)			'Now save the mp3
		
				#IF B4A
					Dim out As OutputStream = File.OpenOutput(File.DirInternal, "CloudTTS.mp3", False)
				#Else IF B4J
				Dim out As OutputStream = File.OpenOutput(File.DirApp, getFileName(strPrevToSpeak)&" "&LanguageCode&"-"&LanguageName&".mp3", False)
				#End If
				out.WriteBytes(decoded, 0, decoded.Length)
				out.Close
				
				'Now, play the audio
				#IF B4A
					audio.Load(File.DirInternal, "CloudTTS.mp3")
				#Else IF B4J
				audio=AudioClip_Static.NewAudioClip2(File.DirApp, getFileName(strPrevToSpeak)&" "&LanguageCode&"-"&LanguageName&".mp3")
					audio.SetCompleteListener(Me, "audio")
				#End If
				audio.Play
				CallSubDelayed(CloudTTS_Parent, CloudTTS_Event_Name & "_StartedSpeaking")
			Catch
				CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_onError", LastException.Message)
			End Try
		Else
			CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_onError", Job.ErrorMessage)
		End If
	End If
	Job.Release
End Sub









'Speaks the Text
'strText = Text to Speak
Public Sub Speak(strText As String)
	strPrevToSpeak=strText
	Dim mapReq As Map:mapReq.Initialize
		
	mapReq.Put("audioConfig", CreateMap("audioEncoding":"MP3"))
	mapReq.Put("input", CreateMap("text":strText))
	mapReq.Put("voice", CreateMap("languageCode":LanguageCode, "name":LanguageName, "ssmlGender":LanguageGender))
'	mapReq.Put("voice", CreateMap("languageCode":LanguageCode))
	
	Dim json As JSONGenerator:json.Initialize(mapReq)
	
	Dim j As HttpJob:j.Initialize("speak", Me) 'name is empty as it is no longer needed
	j.Tag="speak"
	j.PostString("https://texttospeech.googleapis.com/v1/text:synthesize?key="&API_KEY, json.ToString)
	j.GetRequest.SetContentType("application/json; charset=utf-8")
'	Wait For (j) JobDone(j As HttpJob)
'	If j.Success Then
'		Try
'			Dim jsonParser As JSONParser
'			jsonParser.Initialize(j.GetString)
'			Dim strAudio As String=jsonParser.NextObject.Get("audioContent")
'			
'		    Dim su As StringUtils ' jStringUtils
'		    Dim decoded() As Byte = su.DecodeBase64(strAudio)			'Now save the mp3
'	
'			Dim out As OutputStream = File.OpenOutput(File.DirInternal, "CloudTTS.mp3", False)
'			out.WriteBytes(decoded, 0, decoded.Length)
'			out.Close
'			
'			'Now, play the audio
'			MP.Load(File.DirInternal, "CloudTTS.mp3")
'			MP.Play
'		Catch
'			CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_onError", LastException.Message)
'		End Try
'	Else
'		CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_onError", j.ErrorMessage)
'	End If
'	j.Release
End Sub

'Converts the given text into mp3 and saves it to the desired path
'toSpeak- Text to convert
'fileDir- Directory to save file in
'fileName- Name of the file to save as
'ConvertToMP3("This is the text to save", File.DirInternal, "CloudTTS.mp3")
Sub ConvertToMP3(toSpeak As String, fileDir As String, fileName As String)
	strPrevToSpeak=toSpeak
	Dim mapReq As Map:mapReq.Initialize
		
	mapReq.Put("audioConfig", CreateMap("audioEncoding":"MP3"))
	mapReq.Put("input", CreateMap("text":toSpeak))
	mapReq.Put("voice", CreateMap("languageCode":LanguageCode, "name":LanguageName, "ssmlGender":LanguageGender))
'	mapReq.Put("voice", CreateMap("languageCode":LanguageCode))
	
	Dim json As JSONGenerator:json.Initialize(mapReq)
	
	Dim job As HttpJob:job.Initialize("speak", Me) 'name is empty as it is no longer needed
	job.Tag="speak"
	job.PostString("https://texttospeech.googleapis.com/v1/text:synthesize?key="&API_KEY, json.ToString)
	job.GetRequest.SetContentType("application/json; charset=utf-8")
	Wait For (job) JobDone(job As HttpJob)
		If job.Success Then
			Try
				Dim jsonParser As JSONParser
				jsonParser.Initialize(job.GetString)
				Dim strAudio As String=jsonParser.NextObject.Get("audioContent")
				
			    Dim su As StringUtils ' jStringUtils
			    Dim decoded() As Byte = su.DecodeBase64(strAudio)			'Now save the mp3
		
				Dim out As OutputStream = File.OpenOutput(fileDir, fileName, False)
				out.WriteBytes(decoded, 0, decoded.Length)
				out.Close
			Catch
				CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_onError", LastException.Message)
			End Try
		Else
			CallSubDelayed2(CloudTTS_Parent, CloudTTS_Event_Name & "_onError", job.ErrorMessage)
		End If
End Sub

'Resumes the Speaking, if already speaking
Public Sub ResumeSpeaking
	#IF B4A
		If audio.IsPlaying=False Then audio.Play
	#End If
End Sub
'Pauses the Speaking, if already speaking
Public Sub PauseSpeaking
	#IF B4A
		If audio.IsPlaying Then audio.Pause
	#End If
End Sub
'Stops the Speaking, if already speaking
Public Sub StopSpeaking
	If audio.IsPlaying Then audio.Stop
End Sub

'Calls sub CloudTTS_onComplete
Sub audio_Complete
	CallSubDelayed(CloudTTS_Parent, CloudTTS_Event_Name & "_onComplete")
End Sub


'Gets a string and returns a file name safe to save
Sub getFileName(strTest As String) As String
	Dim strCheck() As String= Array As String("|", "\\", "?", "*", "<", "\", ":", ">", CRLF)
	For i=0 To strCheck.Length-1
		Do While (strTest.Contains(strCheck(i)))
			strTest=strTest.Replace(strTest.SubString2(i,i+1), "")
		Loop
	Next
	Return strTest
End Sub
